<?php

namespace apexl\Io\modules\user\entities;

use apexl\Io\includes\Entity;
use apexl\Io\modules\user\entities\operators\refreshTokenOperator;
use apexl\Io\modules\user\entities\operators\userContactDetailsOperator;
use apexl\Io\modules\user\entities\operators\userNoteOperator;
use apexl\Io\operators\entityDatabaseOperator;

class userContactDetailsEntity extends Entity
{
    public function __construct()
    {
        parent::__construct('user_contact_details');
        $this->setOperator(new userContactDetailsOperator('user_contact_details', 'user_id'));
    }
}
